/**
 * OW2 Util
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SshProtocol.java 5684 2010-11-16 22:10:54Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.util.protocol.ssh.impl.jsch;

import java.util.Map;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.protocol.api.IProtocol;
import org.ow2.util.protocol.api.ProtocolConfig;
import org.ow2.util.protocol.api.ProtocolException;
import org.ow2.util.protocol.ssh.api.SshProtocolConfig;

import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.ProxyHTTP;
import com.jcraft.jsch.ProxySOCKS5;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UserInfo;

/**
 *
 * @author Loris Bouzonnet
 */
public class SshProtocol implements IProtocol {

    public static final int DEFAULT_SSH_PORT = 22;

    private static Log logger = LogFactory.getLog(SshProtocol.class);

    public SshProtocolSession createSession(final ProtocolConfig config)
    throws ProtocolException {

        final JSch jsch = new JSch();

        try {
            if (config.getPort() == null) {
                config.setPort(DEFAULT_SSH_PORT);
            }

            final Session session =
                jsch.getSession(config.getUser(), config.getHost(), config.getPort());

            String privateKey = null;
            String passphrase = null;

            if (config instanceof SshProtocolConfig) {

                final SshProtocolConfig sshProtocolConfig = (SshProtocolConfig) config;

                privateKey = sshProtocolConfig.getPrivateKeyPath();
                passphrase = sshProtocolConfig.getPassphrase();

                if (privateKey != null) {
                    jsch.addIdentity(
                            privateKey,
                            passphrase);
                }

                if (sshProtocolConfig.getKnownHostsPath() != null) {
                    jsch.setKnownHosts(sshProtocolConfig.getKnownHostsPath());
                }

                if (sshProtocolConfig.getCompressionLevel() != null) {
                    session.setConfig("compression.s2c", "zlib@openssh.com,zlib,none");
                    session.setConfig("compression.c2s", "zlib@openssh.com,zlib,none");
                    session.setConfig("compression_level",
                            sshProtocolConfig.getCompressionLevel().toString());
                }

                final String proxyHttp = sshProtocolConfig.getProxyHttp();
                final String proxySocks4 = sshProtocolConfig.getProxySocks4();
                final String proxySocks5 = sshProtocolConfig.getProxySocks5();
                if (proxyHttp != null && proxySocks4 != null ||
                        proxyHttp != null && proxySocks5 != null ||
                        proxySocks4 != null && proxySocks5 != null) {
                    logger.warn("Cannot simultaneously use many proxies: HTTP will be preferred.");
                }

                if (proxyHttp != null) {
                    session.setProxy(new ProxyHTTP(proxyHttp));
                } else if (proxySocks4 != null) {
                    session.setProxy(new ProxySOCKS5(proxySocks4));
                } else if (proxySocks5 != null) {
                    session.setProxy(new ProxySOCKS5(proxySocks5));
                }

                final String x11Forwarding = sshProtocolConfig.getX11Forwarding();
                if (x11Forwarding != null) {
                    final String[] hostPort = x11Forwarding.split(":");
                    session.setX11Host(hostPort[0]);
                    if (hostPort.length == 2) {
                        session.setX11Port(Integer.parseInt(hostPort[1]) + 6000);
                    }
                }

                if (sshProtocolConfig.getConnectionTimeout() != null) {
                    session.setTimeout(sshProtocolConfig.getConnectionTimeout());
                }

            }

            // username and password will be given via UserInfo interface.
            final UserInfo ui;
            if (config.getPassword() == null && privateKey == null) {
                ui= new InteractiveUserInfo();
            } else {
                ui = new CachedUserInfo(config.getPassword(), passphrase);
            }
            session.setUserInfo(ui);
            session.connect();

            return new SshProtocolSession(session, config);

        } catch (final JSchException e) {
            throw new ProtocolException(e);
        }
    }

    public ProtocolConfig createConfig(final String url, final Map<String, String> context)
    throws ProtocolException {
        return SshProtocolConfigFactory.createSshProtocolConfig(url, context);
    }

}
